package com.usky.fire.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.usky.common.core.bean.CommonPage;
import com.usky.common.mybatis.core.AbstractCrudService;
import com.usky.common.security.utils.SecurityUtils;
import com.usky.fire.domain.AlarmFire;
import com.usky.fire.mapper.AlarmFireMapper;
import com.usky.fire.service.AlarmFireService;
import com.usky.fire.service.util.OnlineMethod;
import com.usky.fire.service.vo.AlarmFireStatisticalVo;
import com.usky.fire.service.vo.AlarmFireVo;
import com.usky.system.model.LoginUser;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.*;

/**
 * <p>
 * 火灾报警系统报警信息表 服务实现类
 * </p>
 *
 * @author JCB
 * @since 2022-07-29
 */
@Service
public class AlarmFireServiceImpl extends AbstractCrudService<AlarmFireMapper, AlarmFire> implements AlarmFireService {

    @Override
    public CommonPage<AlarmFireVo> alarmFireLowerList(Integer handlingStatus, String startDate, String endDate,
                                                      Integer pageNum, Integer pageSize, String alarmType) {
        DateTimeFormatter df = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");
        List<Integer> alarmTypeList = new ArrayList<>();
        if (alarmType != null && !"".equals(alarmType)) {
            String[] alarmTypeArray = alarmType.split(",");
            for (int i = 0; i < alarmTypeArray.length; i++) {
                alarmTypeList.add(Integer.parseInt(alarmTypeArray[i]));
            }
        }
        String userType = null;
        LoginUser loginUser = SecurityUtils.getLoginUser();
        if (loginUser != null && !"".equals(loginUser)) {
            userType = loginUser.getUserType();
        }
        //缺少维保单位部分代码
        List<String> devcieCodeList = new ArrayList<>();
        CommonPage<AlarmFire> alarmFireList = this.alarmFireList(devcieCodeList, handlingStatus, startDate, endDate, pageNum, pageSize, alarmTypeList);
        List<AlarmFireVo> list = new ArrayList<>();
        for (int i = 0; i < alarmFireList.getRecords().size(); i++) {
            AlarmFireVo alarmFireVo = new AlarmFireVo();
            alarmFireVo.setId(alarmFireList.getRecords().get(i).getId());
            alarmFireVo.setDeviceCode(alarmFireList.getRecords().get(i).getDeviceCode());
            alarmFireVo.setAlarmType(alarmFireList.getRecords().get(i).getAlarmType());
            alarmFireVo.setAlarmTime(df.format(alarmFireList.getRecords().get(i).getAlarmTime()));
            alarmFireVo.setAddress(alarmFireList.getRecords().get(i).getAddress());
            alarmFireVo.setAlarmStatus(alarmFireList.getRecords().get(i).getAlarmStatus());
            alarmFireVo.setOriginalType(alarmFireList.getRecords().get(i).getOriginalType());
            alarmFireVo.setAlarmName(alarmFireList.getRecords().get(i).getAlarmName());
            alarmFireVo.setOriginalPoint(alarmFireList.getRecords().get(i).getOriginalPoint());
            alarmFireVo.setPointType(alarmFireList.getRecords().get(i).getPointType());
            alarmFireVo.setPointDescribe(alarmFireList.getRecords().get(i).getPointDescribe());
            alarmFireVo.setPortNum(alarmFireList.getRecords().get(i).getPortNum());
            alarmFireVo.setHostNum(alarmFireList.getRecords().get(i).getHostNum());
            alarmFireVo.setLoopNum(alarmFireList.getRecords().get(i).getLoopNum());
            alarmFireVo.setPtNum(alarmFireList.getRecords().get(i).getPtNum());
            alarmFireVo.setHandler(alarmFireList.getRecords().get(i).getHandler());
            if (alarmFireList.getRecords().get(i).getHandlingTime()!=null){
                alarmFireVo.setHandlingTime(df.format(alarmFireList.getRecords().get(i).getHandlingTime()));
            }else {
                alarmFireVo.setHandlingTime(null);
            }
            alarmFireVo.setHandlingContent(alarmFireList.getRecords().get(i).getHandlingContent());
            alarmFireVo.setHandlerPhone(alarmFireList.getRecords().get(i).getHandlerPhone());
            alarmFireVo.setHandlingStatus(alarmFireList.getRecords().get(i).getHandlingStatus());
            alarmFireVo.setFalseAlarm(alarmFireList.getRecords().get(i).getFalseAlarm());
            alarmFireVo.setSitePhoto(alarmFireList.getRecords().get(i).getSitePhoto());
            alarmFireVo.setCompanyName("未定义单位表");
            list.add(alarmFireVo);
        }
        return new CommonPage<>(list, alarmFireList.getTotal(), pageSize, pageNum);
    }


    @Override
    public Map<String, Object> fireStatistics(String alarmType) {

        List<Integer> alarmTypeList = new ArrayList<>();
        if (alarmType != null && !"".equals(alarmType)) {
            String[] alarmTypeArray = alarmType.split(",");
            for (int i = 0; i < alarmTypeArray.length; i++) {
                alarmTypeList.add(Integer.parseInt(alarmTypeArray[i]));
            }
        }

        String userType = null;
        LoginUser loginUser = SecurityUtils.getLoginUser();
        if (loginUser != null && !"".equals(loginUser)) {
            userType = loginUser.getUserType();
        }
        //缺少维保单位部分代码
        List<String> devcieCodeList = new ArrayList<>();

        int untreated = this.fireCount(devcieCodeList, 0, alarmTypeList);
        int processed = this.fireCount(devcieCodeList, 1, alarmTypeList);
//        String disposalRate =  processed / (untreated + processed) + "%";
        Integer count = untreated + processed;
        String disposalRate = OnlineMethod.myPercent(processed, count);
        Map<String, Object> map = new HashMap<>();
        map.put("untreated", untreated);//告警未处理数
        map.put("processed", processed);//告警未处理数
        map.put("disposalRate", disposalRate);//告警处理率
        map.put("alarmCount", count);//告警总数
//        map.put("deviceCount", 0);//设备总数
//        map.put("deviceOfflineCount", 0);//设备离线数
//        map.put("deviceAlarmCount", 0);//设备告警数
//        map.put("deviceNormalCount", 0);//设备正常数
//        map.put("deviceOfflineRate", 0);//设备离线率
//        map.put("deviceAlarmRate", 0);//设备告警率
//        map.put("deviceOfflineRate", 0);//设备正常率
        return map;
    }

    @Override
    public Map<String, Object> fireStatisticalChart(String alarmType, String startDate, String endDate) {
        List<Integer> alarmTypeList = new ArrayList<>();
        if (alarmType != null && !"".equals(alarmType)) {
            String[] alarmTypeArray = alarmType.split(",");
            for (int i = 0; i < alarmTypeArray.length; i++) {
                alarmTypeList.add(Integer.parseInt(alarmTypeArray[i]));
            }
        }

        String userType = null;
        LoginUser loginUser = SecurityUtils.getLoginUser();
        if (loginUser != null && !"".equals(loginUser)) {
            userType = loginUser.getUserType();
        }
        //缺少维保单位部分代码
        List<String> devcieCodeList = new ArrayList<>();
        List<AlarmFireStatisticalVo> list = baseMapper.selectAlarmFireStatistical(devcieCodeList, alarmTypeList, startDate, endDate);
        List<Integer> alarmCountList = new ArrayList<>();
        List<Integer> handleCountList = new ArrayList<>();
        List<String> alarmTimeList = new ArrayList<>();
        List<String> handleRateList = new ArrayList<>();
        for (int i = 0; i < list.size(); i++) {
            alarmCountList.add(list.get(i).getCount());
            handleCountList.add(list.get(i).getSum());
            alarmTimeList.add(list.get(i).getAlarmTime());
            handleRateList.add(OnlineMethod.myPercent(list.get(i).getSum(), list.get(i).getCount()));
        }
        Map<String, Object> map = new HashMap<>();
        map.put("alarmCountList", alarmCountList);
        map.put("handleCountList", handleCountList);
        map.put("alarmTimeList", alarmTimeList);
        map.put("handleRateList", handleRateList);
        return map;
    }


    @Override
    public void updateAlarmFire(AlarmFire alarmFire) {
        alarmFire.setHandler(SecurityUtils.getUsername());
        alarmFire.setHandlingTime(LocalDateTime.now());
        alarmFire.setHandlingStatus(1);
        this.updateById(alarmFire);
    }



    @Override
    public List<AlarmFireVo> alarmFireLowerListExport(Integer handlingStatus, String startDate, String endDate, String alarmType) {
        DateTimeFormatter df = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");
        List<Integer> alarmTypeList = new ArrayList<>();
        if (alarmType != null && !"".equals(alarmType)) {
            String[] alarmTypeArray = alarmType.split(",");
            for (int i = 0; i < alarmTypeArray.length; i++) {
                alarmTypeList.add(Integer.parseInt(alarmTypeArray[i]));
            }
        }
        String userType = null;
        LoginUser loginUser = SecurityUtils.getLoginUser();
        if (loginUser != null && !"".equals(loginUser)) {
            userType = loginUser.getUserType();
        }
        //缺少维保单位部分代码
        List<String> devcieCodeList = new ArrayList<>();
        CommonPage<AlarmFire> alarmFireList = this.alarmFireList(devcieCodeList, handlingStatus, startDate, endDate, 0, 0, alarmTypeList);
        List<AlarmFireVo> list = new ArrayList<>();
        for (int i = 0; i < alarmFireList.getRecords().size(); i++) {
            AlarmFireVo alarmFireVo = new AlarmFireVo();
            alarmFireVo.setXh(i + 1);
            alarmFireVo.setId(alarmFireList.getRecords().get(i).getId());
            alarmFireVo.setDeviceCode(alarmFireList.getRecords().get(i).getDeviceCode());
            alarmFireVo.setAlarmType(alarmFireList.getRecords().get(i).getAlarmType());
            alarmFireVo.setAlarmTime(df.format(alarmFireList.getRecords().get(i).getAlarmTime()));
            alarmFireVo.setAddress(alarmFireList.getRecords().get(i).getAddress());
            alarmFireVo.setAlarmStatus(alarmFireList.getRecords().get(i).getAlarmStatus());
            alarmFireVo.setOriginalType(alarmFireList.getRecords().get(i).getOriginalType());
            alarmFireVo.setAlarmName(alarmFireList.getRecords().get(i).getAlarmName());
            alarmFireVo.setOriginalPoint(alarmFireList.getRecords().get(i).getOriginalPoint());
            alarmFireVo.setPointType(alarmFireList.getRecords().get(i).getPointType());
            alarmFireVo.setPointDescribe(alarmFireList.getRecords().get(i).getPointDescribe());
            alarmFireVo.setPortNum(alarmFireList.getRecords().get(i).getPortNum());
            alarmFireVo.setHostNum(alarmFireList.getRecords().get(i).getHostNum());
            alarmFireVo.setLoopNum(alarmFireList.getRecords().get(i).getLoopNum());
            alarmFireVo.setPtNum(alarmFireList.getRecords().get(i).getPtNum());
            alarmFireVo.setHandler(alarmFireList.getRecords().get(i).getHandler());
            if (alarmFireList.getRecords().get(i).getHandlingTime()!=null){
                alarmFireVo.setHandlingTime(df.format(alarmFireList.getRecords().get(i).getHandlingTime()));
            }else {
                alarmFireVo.setHandlingTime(null);
            }
            alarmFireVo.setHandlingContent(alarmFireList.getRecords().get(i).getHandlingContent());
            alarmFireVo.setHandlerPhone(alarmFireList.getRecords().get(i).getHandlerPhone());
            alarmFireVo.setHandlingStatus(alarmFireList.getRecords().get(i).getHandlingStatus());
            alarmFireVo.setFalseAlarm(alarmFireList.getRecords().get(i).getFalseAlarm());
            alarmFireVo.setSitePhoto(alarmFireList.getRecords().get(i).getSitePhoto());
            alarmFireVo.setCompanyName("未定义单位表");
            list.add(alarmFireVo);
        }
        return list;
    }


    /**
     * 火警告警处置/故障告警处置-数量统计
     *
     * @param deviceCodeList 单位编号
     * @param handlingStatus 处理状态（0、未处理 1、已处理）
     * @param alarmTypeList  告警类型（2 火警、4 故障、16 监管等）
     * @return
     */
    @Override
    public Integer fireCount(List<String> deviceCodeList, Integer handlingStatus, List<Integer> alarmTypeList) {
        LambdaQueryWrapper<AlarmFire> queryWrapper = Wrappers.lambdaQuery();
        if (deviceCodeList.size() > 0) {
            queryWrapper.in(AlarmFire::getDeviceCode, deviceCodeList);
        }
        if (handlingStatus != null) {
            queryWrapper.eq(AlarmFire::getHandlingStatus, handlingStatus);
        }
        if (alarmTypeList.size() > 0) {
            if (alarmTypeList.contains(100)){
                List<Integer> alarmTypeList1 = new ArrayList<>();
                alarmTypeList1.add(2);
                alarmTypeList1.add(4);
                queryWrapper.notIn(AlarmFire::getAlarmType, alarmTypeList1);
            }else {
                queryWrapper.in(AlarmFire::getAlarmType, alarmTypeList);
            }
        }
        int total = this.count(queryWrapper);
        return total;
    }

    /**
     * 火警告警处置/故障告警处置-列表数据查询
     *
     * @param deviceCodeList 设备编号
     * @param handlingStatus 处理状态（0、未处理 1、已处理）
     * @param startDate      开始时间 格式：yyyy-MM-dd HH:mm:ss
     * @param endDate        结束时间 格式：yyyy-MM-dd HH:mm:ss
     * @param pageNum        当前页
     * @param pageSize       每页条数
     * @param alarmTypeList  告警类型（2 火警、4 故障、16 监管等、100隐患）
     * @return
     */
    @Override
    public CommonPage<AlarmFire> alarmFireList(List<String> deviceCodeList, Integer handlingStatus, String startDate,
                                               String endDate, Integer pageNum, Integer pageSize, List<Integer> alarmTypeList) {
        LambdaQueryWrapper<AlarmFire> queryWrapper = Wrappers.lambdaQuery();
        if (deviceCodeList.size() > 0) {
            queryWrapper.in(AlarmFire::getDeviceCode, deviceCodeList);
        }
        if (handlingStatus != null) {
            queryWrapper.eq(AlarmFire::getHandlingStatus, handlingStatus);
        }

        if (alarmTypeList.size() > 0) {
            if (alarmTypeList.contains(100)){
                List<Integer> alarmTypeList1 = new ArrayList<>();
                alarmTypeList1.add(2);
                alarmTypeList1.add(4);
                queryWrapper.notIn(AlarmFire::getAlarmType, alarmTypeList1);
            }else {
                queryWrapper.in(AlarmFire::getAlarmType, alarmTypeList);
            }
        }

        if (startDate != null && !"".equals(startDate) && endDate != null && !"".equals(endDate)) {
            queryWrapper.between(AlarmFire::getAlarmTime, startDate, endDate);
        }
        int total = this.count(queryWrapper);
        queryWrapper.orderByDesc(AlarmFire::getId);
        if (pageNum != null && pageNum != 0 && pageSize != null && pageSize != 0) {
            Integer startFate = OnlineMethod.getStartFate(pageNum, pageSize);
            queryWrapper.last("limit " + startFate + "," + pageSize);
        }
        List<AlarmFire> list = this.list(queryWrapper);
        return new CommonPage<>(list, total, pageSize, pageNum);
    }
}
